@@ -1,5 +1,5 @@
 /***************************************************************************
- *   2007-2021 by Peter Semiletov                                          *
+ *   2007-2022 by Peter Semiletov                                          *
  *   peter.semiletov@gmail.com                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -50,7 +50,7 @@ code from qwriter:
 #include <QDebug>
 #include <QUrl>
 #include <QDir>
-#include <QXmlStreamReader>
+//#include <QXmlStreamReader>
 #include <QMimeData>
 #include <QFile>
 #include <QMessageBox>
@@ -62,6 +62,9 @@ code from qwriter:
 #include "gui_utils.h"
 #include "textproc.h"
 
+#include "pugixml.hpp"
+
+
 #define SK_A 38
 #define SK_D 40
 #define SK_W 25
@@ -142,10 +145,10 @@ CSyntaxHighlighterQRegExp::CSyntaxHighlighterQRegExp (QTextDocument *parent, CDo
                                                       CSyntaxHighlighter (parent, doc, fname)
 {
   document = doc;
-  load_from_xml (fname);
+  load_from_xml_pugi (fname);
 }
 
-
+/*
 void CSyntaxHighlighterQRegExp::load_from_xml (const QString &fname)
 {
   if (! file_exists (fname))
@@ -172,11 +175,11 @@ void CSyntaxHighlighterQRegExp::load_from_xml (const QString &fname)
 
          if (xml.isStartElement())
             {
-             /*if (tag_name == "document")
-                {
-                 exts = xml.attributes().value ("exts").toString();
-                 langs = xml.attributes().value ("langs").toString();
-                }*/
+             //if (tag_name == "document")
+               // {
+               //  exts = xml.attributes().value ("exts").toString();
+               //  langs = xml.attributes().value ("langs").toString();
+              //  }
 
              if (tag_name == "item")
                 {
@@ -228,7 +231,7 @@ void CSyntaxHighlighterQRegExp::load_from_xml (const QString &fname)
                  if (attr_type == "mcomment-start")
                     {
                      QString color = hash_get_val (global_palette, xml.attributes().value ("color").toString(), "gray");
-                     QTextCharFormat fmt = tformat_from_style (xml.attributes().value ("color").toString(), color, darker_val);
+                     QTextCharFormat fmt = tformat_from_style (xml.attributes().value ("fontstyle").toString(), color, darker_val);
 
                      fmt_multi_line_comment = fmt;
                      QString element = xml.readElementText().trimmed().remove('\n');
@@ -274,7 +277,7 @@ void CSyntaxHighlighterQRegExp::load_from_xml (const QString &fname)
 
   } //cycle
 }
-
+*/
 
 void CSyntaxHighlighterQRegExp::highlightBlock (const QString &text)
 {
@@ -346,6 +349,186 @@ void CSyntaxHighlighterQRegExp::highlightBlock (const QString &text)
         }
 }
 
+
+class CXMLHL_walker: public pugi::xml_tree_walker
+{
+public:
+
+  CSyntaxHighlighterQRegExp *hl;
+  int darker_val;
+
+
+  bool for_each (pugi::xml_node& node);
+};
+
+
+bool CXMLHL_walker::for_each (pugi::xml_node &node)
+{
+  if (node.type() != pugi::node_element)
+      return true;
+
+  QString tag_name = node.name();
+
+  if (tag_name == "item")
+     {
+      pugi::xml_attribute attr = node.attribute ("type");
+      QString attr_type = attr.as_string();
+      attr = node.attribute ("name");
+      QString attr_name = attr.as_string();
+
+      if (attr_name == "options")
+         {
+          attr = node.attribute ("casecare");
+
+          QString s_casecare = attr.as_string();
+          if (! s_casecare.isEmpty())
+             if (s_casecare == "0" || s_casecare == "false")
+                hl->casecare = false;
+
+          if (! hl->casecare)
+              hl->cs = Qt::CaseInsensitive;
+         }
+
+      if (attr_type == "keywords")
+         {
+          attr = node.attribute ("color");
+          QString color = hash_get_val (global_palette, attr.as_string(), "darkBlue");
+
+          attr = node.attribute ("fontstyle");
+          QTextCharFormat fmt = tformat_from_style (attr.as_string(), color, darker_val);
+
+          QString t = node.text().as_string();
+          QString element = t.trimmed().remove('\n');
+          if (element.isEmpty())
+             return true;
+
+          QRegExp rg (element, hl->cs);
+
+          if (! rg.isValid())
+             qDebug() << "! valid " << rg.pattern();
+          else
+              hl->hl_rules.push_back (make_pair (rg, fmt));
+
+          } //keywords
+      else
+      if (attr_type == "item")
+         {
+          attr = node.attribute ("color");
+          QString color = hash_get_val (global_palette, attr.as_string(), "darkBlue");
+
+          attr = node.attribute ("fontstyle");
+          QTextCharFormat fmt = tformat_from_style (attr.as_string(), color, darker_val);
+
+          QString t = node.text().as_string();
+          QString element = t.trimmed().remove('\n');
+          if (element.isEmpty())
+             return true;
+
+          QRegExp rg (element, hl->cs);
+          if (rg.isValid())
+              hl->hl_rules.push_back (make_pair (rg, fmt));
+          }
+       else
+       if (attr_type == "mcomment-start")
+          {
+           attr = node.attribute ("color");
+
+           QString color = hash_get_val (global_palette, attr.as_string(), "gray");
+
+           attr = node.attribute ("fontstyle");
+
+           QString fontstyle = attr.as_string();
+           QTextCharFormat fmt = tformat_from_style (fontstyle, color, darker_val);
+
+           hl->fmt_multi_line_comment = fmt;
+
+           QString t = node.text().as_string();
+           QString element = t.trimmed().remove('\n');
+           if (element.isEmpty())
+              return true;
+
+           if (! element.isEmpty())
+              {
+               hl->comment_start_expr.first = QRegExp (element, hl->cs, QRegExp::RegExp);
+               hl->comment_start_expr.second = true;
+             }
+          }
+       else
+       if (attr_type == "mcomment-end")
+          {
+           QString t = node.text().as_string();
+           QString element = t.trimmed().remove('\n');
+           if (element.isEmpty())
+              return true;
+
+           if (! element.isEmpty())
+             {
+              hl->comment_end_expr.first = QRegExp (element, hl->cs, QRegExp::RegExp);
+              hl->comment_end_expr.second = true;
+             }
+          }
+       else
+       if (attr_type == "comment")
+          {
+           attr = node.attribute ("name");
+           QString name = attr.as_string();
+
+           QString t = node.text().as_string();
+           QString element = t.trimmed().remove('\n');
+           if (element.isEmpty())
+              return true;
+
+           if (name == "cm_mult")
+               hl->comment_mult = element;
+           else
+           if (name == "cm_single")
+               hl->comment_single = element;
+          }
+
+      }//item
+
+  return true;
+}
+
+
+void CSyntaxHighlighterQRegExp::load_from_xml_pugi (const QString &fname)
+{
+  if (! file_exists (fname))
+     return;
+
+  cs = Qt::CaseSensitive;
+
+  comment_start_expr = make_pair (QRegExp(), false);
+  comment_end_expr = make_pair (QRegExp(), false);
+
+  QString temp = qstring_load (fname);
+  if (temp.isEmpty())
+     return;
+
+  int darker_val = settings->value ("darker_val", 100).toInt();
+
+
+  pugi::xml_document doc;
+  pugi::xml_parse_result result = doc.load_buffer (temp.utf16(),
+                                                   temp.size() * 2,
+                                                   pugi::parse_default,
+                                                   pugi::encoding_utf16);
+
+
+   //pugi::xml_parse_result result = doc.load_buffer (temp.toUtf8().data(),
+     //                                               temp.toUtf8().size());
+
+   if (! result)
+      return;
+
+   CXMLHL_walker walker;
+   walker.darker_val = darker_val;
+   walker.hl = this;
+
+   doc.traverse (walker);
+
+}
+
 #endif
 
 
@@ -355,10 +538,10 @@ CSyntaxHighlighterQRegularExpression::CSyntaxHighlighterQRegularExpression (QTex
                                                                             CSyntaxHighlighter (parent, doc, fname)
 {
   document = doc;
-  load_from_xml (fname);
+  load_from_xml_pugi (fname);
 }
 
-
+/*
 void CSyntaxHighlighterQRegularExpression::load_from_xml (const QString &fname)
 {
   if (! file_exists (fname))
@@ -438,7 +621,7 @@ void CSyntaxHighlighterQRegularExpression::load_from_xml (const QString &fname)
                   if (attr_type == "mcomment-start")
                      {
                       QString color = hash_get_val (global_palette, xml.attributes().value ("color").toString(), "gray");
-                      QTextCharFormat fmt = tformat_from_style (xml.attributes().value ("color").toString(), color, darker_val);
+                      QTextCharFormat fmt = tformat_from_style (xml.attributes().value ("fontstyle").toString(), color, darker_val);
 
                       fmt_multi_line_comment = fmt;
 
@@ -491,6 +674,188 @@ void CSyntaxHighlighterQRegularExpression::load_from_xml (const QString &fname)
 
   } //cycle
 }
+*/
+
+
+class CXMLHL_walker: public pugi::xml_tree_walker
+{
+public:
+
+  CSyntaxHighlighterQRegularExpression *hl;
+  int darker_val;
+
+  bool for_each (pugi::xml_node& node);
+};
+
+
+bool CXMLHL_walker::for_each (pugi::xml_node &node)
+{
+  if (node.type() != pugi::node_element)
+      return true;
+
+//  std::cout << "for_each name = " << node.name() << std::endl;
+
+  QString tag_name = node.name();
+
+  if (tag_name == "item")
+     {
+
+      pugi::xml_attribute attr = node.attribute ("type");
+      QString attr_type = attr.as_string();
+      attr = node.attribute ("name");
+      QString attr_name = attr.as_string();
+
+      if (attr_name == "options")
+         {
+//          attr = node.attribute ("casecare");
+
+          QString s_casecare = node.attribute ("casecare").as_string();
+          if (! s_casecare.isEmpty())
+             if (s_casecare == "0" || s_casecare == "false")
+                hl->casecare = false;
+
+          if (! hl->casecare)
+             hl->pattern_opts = hl->pattern_opts | QRegularExpression::CaseInsensitiveOption;
+         }
+
+      if (attr_type == "keywords")
+         {
+          //attr = node.attribute ("color");
+          QString color = hash_get_val (global_palette, node.attribute ("color").as_string(), "darkBlue");
+
+          //attr = node.attribute ("fontstyle");
+          QTextCharFormat fmt = tformat_from_style (node.attribute ("fontstyle").as_string(), color, darker_val);
+
+          QString t = node.text().as_string();
+          QString element = t.trimmed().remove('\n');
+          if (element.isEmpty())
+             return true;
+
+          QRegularExpression rg = QRegularExpression (element, hl->pattern_opts);
+
+          if (! rg.isValid())
+             qDebug() << "! valid " << rg.pattern();
+          else
+              hl->hl_rules.push_back (make_pair (rg, fmt));
+
+          } //keywords
+      else
+      if (attr_type == "item")
+         {
+//          attr = node.attribute ("color");
+          QString color = hash_get_val (global_palette, node.attribute ("color").as_string(), "darkBlue");
+
+//          attr = node.attribute ("fontstyle");
+          QTextCharFormat fmt = tformat_from_style (node.attribute ("fontstyle").as_string(), color, darker_val);
+
+          QString t = node.text().as_string();
+          QString element = t.trimmed().remove('\n');
+          if (element.isEmpty())
+             return true;
+
+          QRegularExpression rg = QRegularExpression (element, hl->pattern_opts);
+          if (! rg.isValid())
+             qDebug() << "! valid " << rg.pattern();
+          else
+              hl->hl_rules.push_back (make_pair (rg, fmt));
+         }
+       else
+       if (attr_type == "mcomment-start")
+          {
+  //         attr = node.attribute ("color");
+           QString color = hash_get_val (global_palette, node.attribute ("color").as_string(), "gray");
+
+           //attr = node.attribute ("fontstyle");
+           QString fontstyle = node.attribute ("fontstyle").as_string();
+           QTextCharFormat fmt = tformat_from_style (fontstyle, color, darker_val);
+
+           hl->fmt_multi_line_comment = fmt;
+
+           QString t = node.text().as_string();
+           QString element = t.trimmed().remove('\n');
+           if (element.isEmpty())
+              return true;
+
+           QRegularExpression rg = QRegularExpression (element, hl->pattern_opts);
+           if (! rg.isValid())
+              qDebug() << "! valid " << rg.pattern();
+           else
+               {
+                hl->comment_start_expr.first = rg;
+                hl->comment_start_expr.second = true;
+               }
+          }
+       else
+       if (attr_type == "mcomment-end")
+          {
+           QString t = node.text().as_string();
+           QString element = t.trimmed().remove('\n');
+           if (element.isEmpty())
+              return true;
+
+           QRegularExpression rg = QRegularExpression (element, hl->pattern_opts);
+           if (! rg.isValid())
+              qDebug() << "! valid " << rg.pattern();
+           else
+               {
+                hl->comment_end_expr.first = rg;
+                hl->comment_end_expr.second = true;
+               }
+          }
+       else
+       if (attr_type == "comment")
+          {
+           //attr = node.attribute ("name");
+           QString name = node.attribute ("name").as_string();
+
+           QString t = node.text().as_string();
+           QString element = t.trimmed().remove('\n');
+           if (element.isEmpty())
+              return true;
+
+           if (name == "cm_mult")
+               hl->comment_mult = element;
+           else
+           if (name == "cm_single")
+               hl->comment_single = element;
+          }
+
+      }//item
+
+  return true;
+}
+
+
+void CSyntaxHighlighterQRegularExpression::load_from_xml_pugi (const QString &fname)
+{
+  if (! file_exists (fname))
+     return;
+
+  casecare = true;
+
+  comment_start_expr = make_pair (QRegularExpression(), false);
+  comment_end_expr = make_pair (QRegularExpression(), false);
+
+  QString temp = qstring_load (fname);
+  if (temp.isEmpty())
+     return;
+
+  int darker_val = settings->value ("darker_val", 100).toInt();
+
+  pugi::xml_document doc;
+  pugi::xml_parse_result result = doc.load_buffer (temp.utf16(),
+                                                   temp.size() * 2,
+                                                   pugi::parse_default,
+                                                   pugi::encoding_utf16);
+  if (! result)
+     return;
+
+   CXMLHL_walker walker;
+   walker.darker_val = darker_val;
+   walker.hl = this;
+
+   doc.traverse (walker);
+}
 
 
 void CSyntaxHighlighterQRegularExpression::highlightBlock (const QString &text)
@@ -544,7 +909,6 @@ void CSyntaxHighlighterQRegularExpression::highlightBlock (const QString &text)
 
   QRegularExpressionMatch m_start = comment_start_expr.first.match (text);
 
-
   if (previousBlockState() != 1)
       startIndex = m_start.capturedStart();
 
@@ -602,6 +966,7 @@ void CDocument::insertFromMimeData (const QMimeData *source)
   QFileInfo info;
 
   bool b_ins_plain_text = ! source->hasUrls();
+
   if (source->hasUrls() && source->urls().at(0).scheme() != "file")
       b_ins_plain_text = true;
 
@@ -889,7 +1254,6 @@ CDocument::~CDocument()
             file_save_with_name (file_name, charset);
 
 
-
   if (! file_name.startsWith (holder->dir_config) && ! file_name.endsWith (".notes"))
      {
       holder->add_to_recent (this);
@@ -899,7 +1263,9 @@ CDocument::~CDocument()
   if (file_name.startsWith (holder->todo.dir_days))
       holder->todo.load_dayfile();
 
-  QMainWindow *w = qobject_cast <QMainWindow *> (holder->parent_wnd);
+//  QMainWindow *w = qobject_cast <QMainWindow *> (holder->parent_wnd);
+  QMainWindow *w = holder->parent_wnd;
+
   w->setWindowTitle ("");
 
   int i = holder->tab_widget->indexOf (tab_page);
@@ -924,6 +1290,8 @@ bool CDocument::file_open (const QString &fileName, const QString &codec)
 {
   CTio *tio = holder->tio_handler.get_for_fname (fileName);
 
+//  qDebug() << "tio->metaObject()->className()" << tio->metaObject()->className();
+
   if (! tio)
       return false;
 
@@ -959,7 +1327,7 @@ bool CDocument::file_open (const QString &fileName, const QString &codec)
 
   while (i.hasNext())
         {
-         QStringList lt = i.next().split (",");
+         QStringList lt = i.next().split ("*");
          if (lt.size() > 0)
              if (lt.at(0) == file_name)
                 i.remove();
@@ -1049,11 +1417,11 @@ QString CDocument::get_triplex()
      return QString ("");
 
   QString s (file_name);
-  s += ",";
+  s += "*";
   s += charset;
-  s += ",";
+  s += "*";
   s += QString::number (textCursor().position());
-  s += ",";
+  s += "*";
 
   if (! get_word_wrap())
      s+="0";
@@ -1227,6 +1595,7 @@ void CDocument::set_hl (bool mode_auto, const QString &theext)
        }
 
 #else
+
    for (vector<pair<QRegExp, QString> >::iterator p = holder->hl_files.begin(); p != holder->hl_files.end(); ++p)
        {
         if (p->first.isValid())
@@ -1299,7 +1668,9 @@ void CDocument::update_title (bool fullname)
   if (! holder->parent_wnd)
      return;
 
-  QMainWindow *w = qobject_cast <QMainWindow *> (holder->parent_wnd);
+ // QMainWindow *w = qobject_cast <QMainWindow *> (holder->parent_wnd);
+
+  QMainWindow *w = holder->parent_wnd;
 
   if (fullname)
       w->setWindowTitle (file_name);
@@ -1459,10 +1830,10 @@ void CDocument::calc_auto_indent()
      {
       for (int i = 0; i < len; i++)
           if (s.at(i) != '\t')
-            {
-             aindent = i;
-             break;
-            }
+             {
+              aindent = i;
+              break;
+             }
      }
 
 
@@ -2048,11 +2419,11 @@ void CDox::add_to_recent (CDocument *d)
      return;
 
   QString s (d->file_name);
-  s += ",";
+  s += "*";
   s += d->charset;
-  s += ",";
+  s += "*";
   s += QString::number (d->textCursor().position());
-  s += ",";
+  s += "*";
 
   if (! d->get_word_wrap())
      s+="0";
@@ -2164,7 +2535,7 @@ CDocument* CDox::open_file (const QString &fileName, const QString &codec)
 
 CDocument* CDox::open_file_triplex (const QString &triplex)
 {
-  QStringList sl = triplex.split (",");
+  QStringList sl = triplex.split ("*");
   if (sl.size() < 3)
      return 0;
 
@@ -2381,8 +2752,8 @@ void CDox::open_recent()
      return;
 
   CDocument *d = open_file_triplex (recent_files[i]);
-  dir_last = get_file_path (d->file_name);
-
+  if (d)
+      dir_last = get_file_path (d->file_name);
 
   update_recent_menu();
 }
